"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function waitForClientToConnect(client) {
    return new Promise(resolve => {
        client.onConnected(resolve);
    });
}
exports.waitForClientToConnect = waitForClientToConnect;
function execute({ client, extensions, operationName, query, variables, }) {
    return new Promise((resolve, reject) => {
        try {
            let value;
            const subscriber = {
                next(val) {
                    value = val;
                },
                complete() {
                    resolve(value);
                },
                error(err) {
                    reject(err);
                },
            };
            client
                .request({
                extensions,
                operationName,
                query,
                variables,
            })
                .subscribe(subscriber);
        }
        catch (e) {
            reject(e);
        }
    });
}
exports.execute = execute;
function subscribe({ client, extensions, operationName, query, variables, }) {
    const events = [];
    const subscriber = {
        next(event) {
            events.push(event);
        },
        complete() {
            events.push(new Error('Subscription cannot be done'));
        },
        error(err) {
            events.push(err);
        },
    };
    client
        .request({
        extensions,
        operationName,
        query,
        variables,
    })
        .subscribe(subscriber);
    return {
        next() {
            const event = events.shift();
            if (event) {
                if (event instanceof Error) {
                    throw event;
                }
                return { done: false, value: event };
            }
            return { done: true, value: undefined };
        },
    };
}
exports.subscribe = subscribe;
